use std::collections::HashMap;

use banner_pattern::BannerPattern;
use chat_type::ChatType;
use damage_type::DamageType;
use dimension_type::DimensionType;
use painting_variant::PaintingVariant;
use trim_material::TrimMaterial;
use trim_pattern::TrimPattern;
use wolf_variant::WolfVariant;
use worldgen::biome::Biome;

use crate::identifier::Identifier;

pub mod banner_pattern;
pub mod chat_type;
pub mod damage_type;
pub mod dimension_type;
pub mod painting_variant;
pub mod trim_material;
pub mod trim_pattern;
pub mod wolf_variant;
pub mod worldgen;

pub type Registry<T> = HashMap<Identifier, T>;

#[derive(Default)]
pub struct Registries {
    // advancement
    pub banner_patterns: Registry<BannerPattern>,
    pub chat_types: Registry<ChatType>,
    pub damage_types: Registry<DamageType>,
    // dimension
    pub dimension_types: Registry<DimensionType>,
    // enchantment
    // enchantment_provider
    // instrument
    // item_modifier
    // jukebox_song
    // loot_table
    pub painting_variants: Registry<PaintingVariant>,
    // predicate
    // recipe
    // trail_spawner
    pub trim_materials: Registry<TrimMaterial>,
    pub trim_patterns: Registry<TrimPattern>,
    pub wolf_variants: Registry<WolfVariant>,
    pub worldgen: WorldgenRegistries,
}

impl Registries {
    pub fn empty() -> Registries {
        Self::default()
    }
}

#[derive(Default)]
pub struct WorldgenRegistries {
    pub biomes: Registry<Biome>,
    // configured_carver
    // configured_feature
    // density_function
    // noise
    // noise_settings
    // placed_feature
    // processor_list
    // structure
    // structure_set
    // template_pool
    // world_preset
    // flat_level_generator_preset
    // multi_noise_biome_source_parameter_list
}
